# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Compute Availability Set Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
    to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

**cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
import logging

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.compute.models  # pylint: disable=unused-import
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    from msrestazure.tools import is_valid_resource_id
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def create_or_update(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Create or update an availability set.

    :param name: The availability set to create.

    :param resource_group: The resource group name assigned to the
        availability set.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.availability_set.create_or_update testset testgroup

    '''
    if 'location' not in kwargs:
        rg_props = await hub.exec.azurerm.resource.group.get(
            resource_group, **kwargs
        )

        if 'error' in rg_props:
            log.error(
                'Unable to determine location from resource group specified.'
            )
            return False
        kwargs['location'] = rg_props['location']

    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)

    # Use VM names to link to the IDs of existing VMs.
    if isinstance(kwargs.get('virtual_machines'), list):
        vm_list = []
        for vm_name in kwargs.get('virtual_machines'):
            vm_instance = await hub.exec.azurerm.compute.virtual_machine.get(
                name=vm_name,
                resource_group=resource_group,
                **kwargs
            )
            if 'error' not in vm_instance:
                vm_list.append({'id': str(vm_instance['id'])})
        kwargs['virtual_machines'] = vm_list

    try:
        setmodel = await hub.exec.utils.azurerm.create_object_model('compute', 'AvailabilitySet', **kwargs)
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        av_set = compconn.availability_sets.create_or_update(
            resource_group_name=resource_group,
            availability_set_name=name,
            parameters=setmodel
        )
        result = av_set.as_dict()

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def delete(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Delete an availability set.

    :param name: The availability set to delete.

    :param resource_group: The resource group name assigned to the
        availability set.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.availability_set.delete testset testgroup

    '''
    result = False
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        compconn.availability_sets.delete(
            resource_group_name=resource_group,
            availability_set_name=name
        )
        result = True

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)

    return result


async def get(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get a dictionary representing an availability set's properties.

    :param name: The availability set to get.

    :param resource_group: The resource group name assigned to the
        availability set.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.availability_set.get testset testgroup

    '''
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        av_set = compconn.availability_sets.get(
            resource_group_name=resource_group,
            availability_set_name=name
        )
        result = av_set.as_dict()

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_(hub, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all availability sets within a resource group.

    :param resource_group: The resource group name to list availability
        sets within.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.availability_set.list testgroup

    '''
    result = {}
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        avail_sets = await hub.exec.utils.azurerm.paged_object_to_list(
            compconn.availability_sets.list(
                resource_group_name=resource_group
            )
        )

        for avail_set in avail_sets:
            result[avail_set['name']] = avail_set
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_available_sizes(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all available virtual machine sizes that can be used to
    to create a new virtual machine in an existing availability set.

    :param name: The availability set name to list available
        virtual machine sizes within.

    :param resource_group: The resource group name to list available
        availability set sizes within.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.availability_set.list_available_sizes testset testgroup

    '''
    result = {}
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        sizes = await hub.exec.utils.azurerm.paged_object_to_list(
            compconn.availability_sets.list_available_sizes(
                resource_group_name=resource_group,
                availability_set_name=name
            )
        )

        for size in sizes:
            result[size['name']] = size
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Local Network Gateway State Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as a dictionary of
    keyword arguments to the ``connection_auth`` parameter in order to work properly. Since the authentication
    parameters are sensitive, it's recommended to pass them to the states via pillar.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

    **cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud. Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

    Example Pillar for Azure Resource Manager authentication:

    .. code-block:: yaml

        azurerm:
            user_pass_auth:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                username: fletch
                password: 123pass
            mysubscription:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                tenant: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                client_id: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                secret: XXXXXXXXXXXXXXXXXXXXXXXX
                cloud_environment: AZURE_PUBLIC_CLOUD

    Example states using Azure Resource Manager authentication:

    .. code-block:: jinja

        Ensure virtual network exists:
            azurerm.network.virtual_network.present:
                - name: my_vnet
                - resource_group: my_rg
                - address_prefixes:
                    - '10.0.0.0/8'
                    - '192.168.0.0/16'
                - dns_servers:
                    - '8.8.8.8'
                - tags:
                    how_awesome: very
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}

        Ensure virtual network is absent:
            azurerm.network.virtual_network.absent:
                - name: other_vnet
                - resource_group: my_rg
                - connection_auth: {{ profile }}

'''

# Python libs
from __future__ import absolute_import
import logging
import re

try:
    from six.moves import range as six_range
except ImportError:
    six_range = range

log = logging.getLogger(__name__)


async def present(hub, ctx, name, resource_group, gateway_ip_address, bgp_settings=None, address_prefixes=None, tags=None,
            connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a location network gateway exists.

    :param name:
        Name of the local network gateway.

    :param resource_group:
        The resource group assigned to the local network gateway.

    :param gateway_ip_address:
        IP address of local network gateway.

    :param bgp_settings:
        A dictionary representing a valid BgpSettings object, which stores the local network gateway's BGP speaker
        settings. Valid parameters include:
          - ``asn``: The BGP speaker's Autonomous System Number. This is an integer value.
          - ``bgp_peering_address``: The BGP peering address and BGP identifier of this BGP speaker.
                                     This is a string value.
          - ``peer_weight``: The weight added to routes learned from this BGP speaker. This is an integer value.

    :param address_prefixes:
        A list of CIDR blocks which can be used by subnets within the virtual network.
        Represents the local network site address space.

    :param tags:
        A dictionary of strings can be passed as tag metadata to the local network gateway object.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure local network gateway exists:
            azurerm.network.local_network_gateway.present:
                - name: gateway1
                - resource_group: rg-module-testing
                - gateway_ip_address: 192.168.0.1
                - bgp_settings:
                    asn: 65515
                    bgp_peering_address: 10.2.2.2
                    peer_weight: 0
                - address_prefixes:
                    - '10.0.0.0/8'
                    - '192.168.0.0/16'
                - tags:
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    gateway = await hub.exec.azurerm.network.local_network_gateway.get(
        name,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' not in gateway:
        tag_changes = await hub.exec.utils.dictdiffer.deep_diff(gateway.get('tags', {}), tags or {})
        if tag_changes:
            ret['changes']['tags'] = tag_changes

        if gateway_ip_address != gateway.get('gateway_ip_address'):
            ret['changes']['gateway_ip_address'] = {
                'old': gateway.get('gateway_ip_address'),
                'new': gateway_ip_address
            }

        if bgp_settings:
            if not isinstance(bgp_settings, dict):
                ret['comment'] = 'BGP settings must be provided as a dictionary!'
                return ret

            for key in bgp_settings:
                if bgp_settings[key] != gateway.get('bgp_settings', {}).get(key):
                    ret['changes']['bgp_settings'] = {
                        'old': gateway.get('bgp_settings'),
                        'new': bgp_settings
                    }
                    break

        addr_changes = set(address_prefixes or []).symmetric_difference(
            set(gateway.get('local_network_address_space', {}).get('address_prefixes', [])))
        if addr_changes:
            ret['changes']['local_network_address_space'] = {
                'address_prefixes': {
                    'old': gateway.get('local_network_address_space', {}).get('address_prefixes', []),
                    'new': address_prefixes,
                }
            }

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'Local network gateway {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['result'] = None
            ret['comment'] = 'Local network gateway {0} would be updated.'.format(name)
            return ret

    else:
        ret['changes'] = {
            'old': {},
            'new': {
                'name': name,
                'resource_group': resource_group,
                'gateway_ip_address': gateway_ip_address,
                'tags': tags,
            }
        }

        if bgp_settings:
            ret['changes']['new']['bgp_settings'] = bgp_settings
        if address_prefixes:
            ret['changes']['new']['local_network_address_space'] = {'address_prefixes': address_prefixes}

    if ctx['test']:
        ret['comment'] = 'Local network gateway {0} would be created.'.format(name)
        ret['result'] = None
        return ret

    gateway_kwargs = kwargs.copy()
    gateway_kwargs.update(connection_auth)

    gateway = await hub.exec.azurerm.network.local_network_gateway.create_or_update(
        name=name,
        resource_group=resource_group,
        gateway_ip_address=gateway_ip_address,
        local_network_address_space={'address_prefixes': address_prefixes},
        bgp_settings=bgp_settings,
        tags=tags,
        **gateway_kwargs
    )

    if 'error' not in gateway:
        ret['result'] = True
        ret['comment'] = 'Local network gateway {0} has been created.'.format(name)
        return ret

    ret['comment'] = 'Failed to create local network gateway {0}! ({1})'.format(name, gateway.get('error'))
    return ret


async def absent(hub, ctx, name, resource_group, connection_auth=None):
    '''
    .. versionadded:: 1.0.0

    Ensure a local network gateway object does not exist in the resource_group.

    :param name:
        Name of the local network gateway object.

    :param resource_group:
        The resource group associated with the local network gateway.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure local network gateway absent:
            azurerm.network.local_network_gateway.absent:
                - name: gateway1
                - resource_group: group1
                - connection_auth: {{ profile }}
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    gateway = await hub.exec.azurerm.network.local_network_gateway.get(
        name,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' in gateway:
        ret['result'] = True
        ret['comment'] = 'Local network gateway object {0} was not found.'.format(name)
        return ret

    elif ctx['test']:
        ret['comment'] = 'Local network gateway object {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': gateway,
            'new': {},
        }
        return ret

    deleted = await hub.exec.azurerm.network.local_network_gateway.delete(
        name,
        resource_group,
        **connection_auth
    )

    if deleted:
        ret['result'] = True
        ret['comment'] = 'Local network gateway object {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': gateway,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete local network gateway object {0}!'.format(name)
    return ret



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) DNS Zone State Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as a dictionary of
keyword arguments to the ``connection_auth`` parameter in order to work properly. Since the authentication
parameters are sensitive, it's recommended to pass them to the states via pillar.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

    **cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud. Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

    Example Pillar for Azure Resource Manager authentication:

    .. code-block:: yaml

        azurerm:
            user_pass_auth:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                username: fletch
                password: 123pass
            mysubscription:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                tenant: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                client_id: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                secret: XXXXXXXXXXXXXXXXXXXXXXXX
                cloud_environment: AZURE_PUBLIC_CLOUD

    Example states using Azure Resource Manager authentication:

    .. code-block:: yaml

        Ensure DNS zone exists:
            azurerm.dns.zone.present:
                - name: contoso.com
                - resource_group: my_rg
                - tags:
                    how_awesome: very
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}

        Ensure DNS zone is absent:
            azurerm.dns.zone.absent:
                - name: contoso.com
                - resource_group: my_rg
                - connection_auth: {{ profile }}

'''

# Python libs
from __future__ import absolute_import
import logging

import six
try:
    from six.moves import range as six_range
except ImportError:
    six_range = range

log = logging.getLogger(__name__)


async def present(hub, ctx, name, resource_group, etag=None, if_match=None, if_none_match=None, registration_virtual_networks=None,
            resolution_virtual_networks=None, tags=None, zone_type='Public', connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a DNS zone exists.

    :param name:
        Name of the DNS zone (without a terminating dot).

    :param resource_group:
        The resource group assigned to the DNS zone.

    :param etag:
        The etag of the zone. `Etags <https://docs.microsoft.com/en-us/azure/dns/dns-zones-records#etags>`_ are used
        to handle concurrent changes to the same resource safely.

    :param if_match:
        The etag of the DNS zone. Omit this value to always overwrite the current zone. Specify the last-seen etag
        value to prevent accidentally overwritting any concurrent changes.

    :param if_none_match:
        Set to '*' to allow a new DNS zone to be created, but to prevent updating an existing zone. Other values will
        be ignored.

    :param registration_virtual_networks:
        A list of references to virtual networks that register hostnames in this DNS zone. This is only when zone_type
        is Private. (requires `azure-mgmt-dns <https://pypi.python.org/pypi/azure-mgmt-dns>`_ >= 2.0.0rc1)

    :param resolution_virtual_networks:
        A list of references to virtual networks that resolve records in this DNS zone. This is only when zone_type is
        Private. (requires `azure-mgmt-dns <https://pypi.python.org/pypi/azure-mgmt-dns>`_ >= 2.0.0rc1)

    :param tags:
        A dictionary of strings can be passed as tag metadata to the DNS zone object.

    :param zone_type:
        The type of this DNS zone (Public or Private). Possible values include: 'Public', 'Private'. Default value: 'Public'
         (requires `azure-mgmt-dns <https://pypi.python.org/pypi/azure-mgmt-dns>`_ >= 2.0.0rc1)

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure DNS zone exists:
            azurerm.dns.zone.present:
                - name: contoso.com
                - resource_group: my_rg
                - zone_type: Private
                - registration_virtual_networks:
                  - /subscriptions/{{ sub }}/resourceGroups/my_rg/providers/Microsoft.Network/virtualNetworks/test_vnet
                - tags:
                    how_awesome: very
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}

    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    zone = await hub.exec.azurerm.dns.zone.get(name, resource_group, azurearm_log_level='info', **connection_auth)

    if 'error' not in zone:
        tag_changes = await hub.exec.utils.dictdiffer.deep_diff(zone.get('tags', {}), tags or {})
        if tag_changes:
            ret['changes']['tags'] = tag_changes

        # The zone_type parameter is only accessible in azure-mgmt-dns >=2.0.0rc1
        if zone.get('zone_type'):
            if zone.get('zone_type').lower() != zone_type.lower():
                ret['changes']['zone_type'] = {
                    'old': zone['zone_type'],
                    'new': zone_type
                }

            if zone_type.lower() == 'private':
                # The registration_virtual_networks parameter is only accessible in azure-mgmt-dns >=2.0.0rc1
                if registration_virtual_networks and not isinstance(registration_virtual_networks, list):
                    ret['comment'] = 'registration_virtual_networks must be supplied as a list of VNET ID paths!'
                    return ret
                reg_vnets = zone.get('registration_virtual_networks', [])
                remote_reg_vnets = sorted([vnet['id'].lower() for vnet in reg_vnets if 'id' in vnet])
                local_reg_vnets = sorted([vnet.lower() for vnet in registration_virtual_networks or []])
                if local_reg_vnets != remote_reg_vnets:
                    ret['changes']['registration_virtual_networks'] = {
                        'old': remote_reg_vnets,
                        'new': local_reg_vnets
                    }

                # The resolution_virtual_networks parameter is only accessible in azure-mgmt-dns >=2.0.0rc1
                if resolution_virtual_networks and not isinstance(resolution_virtual_networks, list):
                    ret['comment'] = 'resolution_virtual_networks must be supplied as a list of VNET ID paths!'
                    return ret
                res_vnets = zone.get('resolution_virtual_networks', [])
                remote_res_vnets = sorted([vnet['id'].lower() for vnet in res_vnets if 'id' in vnet])
                local_res_vnets = sorted([vnet.lower() for vnet in resolution_virtual_networks or []])
                if local_res_vnets != remote_res_vnets:
                    ret['changes']['resolution_virtual_networks'] = {
                        'old': remote_res_vnets,
                        'new': local_res_vnets
                    }

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'DNS zone {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['result'] = None
            ret['comment'] = 'DNS zone {0} would be updated.'.format(name)
            return ret

    else:
        ret['changes'] = {
            'old': {},
            'new': {
                'name': name,
                'resource_group': resource_group,
                'etag': etag,
                'registration_virtual_networks': registration_virtual_networks,
                'resolution_virtual_networks': resolution_virtual_networks,
                'tags': tags,
                'zone_type': zone_type,
            }
        }

    if ctx['test']:
        ret['comment'] = 'DNS zone {0} would be created.'.format(name)
        ret['result'] = None
        return ret

    zone_kwargs = kwargs.copy()
    zone_kwargs.update(connection_auth)

    zone = await hub.exec.azurerm.dns.zone.create_or_update(
        name=name,
        resource_group=resource_group,
        etag=etag,
        if_match=if_match,
        if_none_match=if_none_match,
        registration_virtual_networks=registration_virtual_networks,
        resolution_virtual_networks=resolution_virtual_networks,
        tags=tags,
        zone_type=zone_type,
        **zone_kwargs
    )

    if 'error' not in zone:
        ret['result'] = True
        ret['comment'] = 'DNS zone {0} has been created.'.format(name)
        return ret

    ret['comment'] = 'Failed to create DNS zone {0}! ({1})'.format(name, zone.get('error'))
    return ret


async def absent(hub, ctx, name, resource_group, connection_auth=None):
    '''
    .. versionadded:: 1.0.0

    Ensure a DNS zone does not exist in the resource group.

    :param name:
        Name of the DNS zone.

    :param resource_group:
        The resource group assigned to the DNS zone.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    zone = await hub.exec.azurerm.dns.zone.get(
        name,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' in zone:
        ret['result'] = True
        ret['comment'] = 'DNS zone {0} was not found.'.format(name)
        return ret

    elif ctx['test']:
        ret['comment'] = 'DNS zone {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': zone,
            'new': {},
        }
        return ret

    deleted = await hub.exec.azurerm.dns.zone.delete(name, resource_group, **connection_auth)

    if deleted:
        ret['result'] = True
        ret['comment'] = 'DNS zone {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': zone,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete DNS zone {0}!'.format(name)
    return ret



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Management Lock Resource Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 2.7.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.34.3
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.2
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
    to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

**cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
from json import loads, dumps
import logging

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.resource.resources.models  # pylint: disable=unused-import
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def create_or_update_at_resource_group_level(hub, name, resource_group, lock_level, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Creates or updates a management lock at the resource group level. When you apply a lock at a parent scope,
        all child resources inherit the same lock. To create management locks, you must have access to
        Microsoft.Authorization/* or Microsoft.Authorization/locks/* actions.

    :param name: The name of the lock. The lock name can be a maximum of 260 characters. It cannot contain
        <, > %, &, :, , ?, /, or any control characters.

    :param resource_group: The name of the resource group.

    :param lock_level: The level of the lock. Possible values are: 'NotSpecified', 'CanNotDelete', & 'ReadOnly'.
        CanNotDelete means authorized users are able to read and modify the resources, but not delete. ReadOnly means
        authorized users can only read from a resource, but they can't modify or delete it.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.create_or_update_at_resource_group_level testname testgroup \
                  testlevel

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        lockmodel = await hub.exec.utils.azurerm.create_object_model(
            'resource.locks',
            'ManagementLockObject',
            level=lock_level,
            **kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        lock = lckconn.management_locks.create_or_update_at_resource_group_level(
            resource_group_name=resource_group,
            lock_name=name,
            parameters=lockmodel
        )

        result = lock.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def delete_at_resource_group_level(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Deletes a management lock at the resource group level. To delete management locks, you must have access to
        Microsoft.Authorization/* or Microsoft.Authorization/locks/* actions.

    :param name: The name of the lock to be deleted.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.delete_at_resource_group_level testname testgroup

    '''
    result = False
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        lock = lckconn.management_locks.delete_at_resource_group_level(
            resource_group_name=resource_group,
            lock_name=name,
            **kwargs
        )

        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def get_at_resource_group_level(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets a management lock at the resource group level.

    :param name: The name of the lock to get.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.get_at_resource_group_level testname testgroup

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        lock = lckconn.management_locks.get_at_resource_group_level(
            resource_group_name=resource_group,
            lock_name=name,
            **kwargs
        )

        result = lock.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def create_or_update_by_scope(hub, name, scope, lock_level, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Create or update a management lock by scope. When you apply a lock at a parent scope,
        all child resources inherit the same lock. To create management locks, you must have access to
        Microsoft.Authorization/* or Microsoft.Authorization/locks/* actions.

    :param name: The name of the lock. The lock name can be a maximum of 260 characters. It cannot contain
        <, > %, &, :, , ?, /, or any control characters.

    :param scope: The scope for the lock. When providing a scope for the assignment,
        use '/subscriptions/{subscriptionId}' for subscriptions,
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}' for resource groups, and
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{parentResourcePathIfPresent}/{resourceType}/{resourceName}'
        for resources.

    :param lock_level: The level of the lock. Possible values are: 'NotSpecified', 'CanNotDelete', & 'ReadOnly'.
        CanNotDelete means authorized users are able to read and modify the resources, but not delete. ReadOnly means
        authorized users can only read from a resource, but they can't modify or delete it.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.create_or_update_by_scope testname testscope testlevel

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        lockmodel = await hub.exec.utils.azurerm.create_object_model(
            'resource.locks',
            'ManagementLockObject',
            level=lock_level,
            **kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        lock = lckconn.management_locks.create_or_update_by_scope(
            scope=scope,
            lock_name=name,
            parameters=lockmodel
        )

        result = lock.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def delete_by_scope(hub, name, scope, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Delete a management lock by scope. To delete management locks, you must have access to
        Microsoft.Authorization/* or Microsoft.Authorization/locks/* actions.

    :param name: The name of the lock to be deleted.

    :param scope: The scope for the lock. When providing a scope for the assignment, 
        use '/subscriptions/{subscriptionId}' for subscriptions, 
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}' for resource groups, and
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{parentResourcePathIfPresent}/{resourceType}/{resourceName}'
        for resources.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.delete_by_scope testname testscope

    '''
    result = False
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        lock = lckconn.management_locks.delete_by_scope(
            scope=scope,
            lock_name=name,
            **kwargs
        )

        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def get_by_scope(hub, name, scope, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get a management lock by scope.

    :param name: The name of the lock to get.

    :param scope: The scope for the lock. When providing a scope for the assignment,
        use '/subscriptions/{subscriptionId}' for subscriptions,
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}' for resource groups, and
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{parentResourcePathIfPresent}/{resourceType}/{resourceName}'
        for resources.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.get_by_scope testname testscope

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        lock = lckconn.management_locks.get_by_scope(
            scope=scope,
            lock_name=name,
            **kwargs
        )

        result = lock.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def create_or_update_at_resource_level(hub, name, lock_level, resource_group, resource, resource_type,
                                             resource_provider_namespace, parent_resource_path=None,
                                             **kwargs):
    '''
    .. versionadded:: 1.0.0

    Creates or updates a management lock at the resource level or any level below the resource. When you apply a lock
        at a parent scope, all child resources inherit the same lock. To create management locks, you must have access
        to Microsoft.Authorization/* or Microsoft.Authorization/locks/* actions.

    :param name: The name of the lock. The lock name can be a maximum of 260 characters. It cannot contain
        <, > %, &, :, , ?, /, or any control characters.

    :param lock_level: The level of the lock. Possible values are: 'NotSpecified', 'CanNotDelete', & 'ReadOnly'.
        CanNotDelete means authorized users are able to read and modify the resources, but not delete. ReadOnly means
        authorized users can only read from a resource, but they can't modify or delete it.

    :param resource_group: The name of the resource group containing the resource to lock.

    :param resource: The name of the resource to lock.

    :param resource_type: The resource type of the resource to lock.

    :param resource_provider_namespace: The resource provider namespace of the resource to lock.

    :param parent_resource_path: The parent resource identity.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.create_or_update_at_resource_level testname testlevel testgroup \
                  testresource testtype testnamespace testpath

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        lockmodel = await hub.exec.utils.azurerm.create_object_model(
            'resource.locks',
            'ManagementLockObject',
            level=lock_level,
            **kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    if parent_resource_path is None:
        parent_resource_path = ''

    try:
        lock = lckconn.management_locks.create_or_update_at_resource_level(
            resource_group_name=resource_group,
            lock_name=name,
            resource_name=resource,
            resource_provider_namespace=resource_provider_namespace,
            resource_type=resource_type,
            parent_resource_path=parent_resource_path,
            parameters=lockmodel
        )

        result = lock.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def delete_at_resource_level(hub, name, resource_group, resource, resource_type, resource_provider_namespace,
                                   parent_resource_path=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Deletes the management lock of a resource or any level below the resource. When you apply a lock
        at a parent scope, all child resources inherit the same lock. To delete management locks, you must have access
        to Microsoft.Authorization/* or Microsoft.Authorization/locks/* actions.

    :param name: The name of the lock to delete.

    :param resource_group: The name of the resource group containing the resource with the lock to delete.

    :param resource: The name of the resource with the lock to delete.

    :param resource_type: The resource type of the resource with the lock to delete.

    :param resource_provider_namespace: The resource provider namespace of the resource with the lock to delete.

    :param parent_resource_path: The parent resource identity.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.delete_at_resource_level testname testgroup testresource \
                  testtype testnamespace testpath

    '''
    result = False
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    if parent_resource_path is None:
        parent_resource_path = ''

    try:
        lock = lckconn.management_locks.delete_at_resource_level(
            lock_name=name,
            resource_group_name=resource_group,
            resource_name=resource,
            resource_provider_namespace=resource_provider_namespace,
            resource_type=resource_type,
            parent_resource_path=parent_resource_path,
            **kwargs
        )

        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def get_at_resource_level(hub, name, resource_group, resource, resource_type, resource_provider_namespace,
                                parent_resource_path=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get the management lock of a resource or any level below resource.

    :param name: The name of the lock.

    :param resource_group: The name of the resource group.

    :param resource: The name of the resource.

    :param resource_type: The type of the resource.

    :param resource_provider_namespace: The namespace of the resource provider.

    :param parent_resource_path: The parent resource identity.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.get_at_resource_level testname testgroup testresource \
                  testtype testnamespace testpath

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    if parent_resource_path is None:
        parent_resource_path = ''

    try:
        lock = lckconn.management_locks.get_at_resource_level(
            lock_name=name,
            resource_group_name=resource_group,
            resource_name=resource,
            resource_provider_namespace=resource_provider_namespace,
            resource_type=resource_type,
            parent_resource_path=parent_resource_path,
            **kwargs
        )

        result = lock.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def create_or_update_at_subscription_level(hub, name, lock_level, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Creates or updates a management lock at the subscription level. When you apply a lock at a parent scope,
        all child resources inherit the same lock. To create management locks, you must have access to
        Microsoft.Authorization/* or Microsoft.Authorization/locks/* actions.

    :param name: The name of the lock. The lock name can be a maximum of 260 characters. It cannot contain
        <, > %, &, :, , ?, /, or any control characters.

    :param lock_level: The level of the lock. Possible values are: 'NotSpecified', 'CanNotDelete', & 'ReadOnly'.
        CanNotDelete means authorized users are able to read and modify the resources, but not delete. ReadOnly means
        authorized users can only read from a resource, but they can't modify or delete it.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.create_or_update_at_subscription_level testname testlevel

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        lockmodel = await hub.exec.utils.azurerm.create_object_model(
            'resource.locks',
            'ManagementLockObject',
            level=lock_level,
            **kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        lock = lckconn.management_locks.create_or_update_at_subscription_level(
            lock_name=name,
            parameters=lockmodel
        )

        result = lock.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def delete_at_subscription_level(hub, name, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Deletes the management lock at the subscription level. To delete management locks, you must have access to
        Microsoft.Authorization/* or Microsoft.Authorization/locks/* actions.

    :param name: The name of the lock to be deleted.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.delete_at_subscription_level testname

    '''
    result = False
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        lock = lckconn.management_locks.delete_at_subscription_level(
            lock_name=name,
            **kwargs
        )

        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def get_at_subscription_level(hub, name, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets a management lock at the subscription level.

    :param name: The name of the lock to get.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.get_at_subscription_level testname

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        lock = lckconn.management_locks.get_at_subscription_level(
            lock_name=name,
            **kwargs
        )

        result = lock.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_at_resource_group_level(hub, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets all the management locks for a resource group.

    :param resource_group: The name of the resource group containing the locks to get.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.list_at_resource_group_level testgroup

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        result = await hub.exec.utils.azurerm.paged_object_to_list(
            lckconn.management_locks.list_at_resource_group_level(
                resource_group_name=resource_group,
                filter=kwargs.get('filter')
            )
        )

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_at_resource_level(hub, resource_group, resource, resource_type, resource_provider_namespace,
                                          parent_resource_path=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get the management lock of a resource or any level below resource.

    :param resource_group: The name of the resource group.

    :param resource: The name of the resource.

    :param resource_type: The type of the resource.

    :param resource_provider_namespace: The namespace of the resource provider.

    :param parent_resource_path: The parent resource identity.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.list_at_resource_level testgroup testresource testtype \
                  testnamespace testpath

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    if parent_resource_path is None:
        parent_resource_path = ''

    try:
        result = await hub.exec.utils.azurerm.paged_object_to_list(
            lckconn.management_locks.list_at_resource_level(
                resource_group_name=resource_group,
                resource_name=resource,
                resource_provider_namespace=resource_provider_namespace,
                resource_type=resource_type,
                parent_resource_path=parent_resource_path,
                filter=kwargs.get('filter')
            )
        )

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_at_subscription_level(hub, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets all the management locks for a subscription.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.list_at_subscription_level

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        result = await hub.exec.utils.azurerm.paged_object_to_list(
            lckconn.management_locks.list_at_subscription_level(
                filter=kwargs.get('filter')
            )
        )

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_by_scope(hub, scope, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets all the management locks for a scope.

    :param scope: The scope for the lock. When providing a scope for the assignment,
        use '/subscriptions/{subscriptionId}' for subscriptions,
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}' for resource groups, and
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{parentResourcePathIfPresent}/{resourceType}/{resourceName}'
        for resources.

    :param scope: The scope for the lock. When providing a scope for the assignment, use '/subscriptions/{subscriptionId}' for subscriptions,
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}' for resource groups, and
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{parentResourcePathIfPresent}/{resourceType}/{resourceName}' for resources.

    CLI Example:

    .. code-block:: bash

        azurerm.resource.management_lock.list_by_scope testscope

    '''
    result = {}
    lckconn = await hub.exec.utils.azurerm.get_client('managementlock', **kwargs)

    try:
        result = await hub.exec.utils.azurerm.paged_object_to_list(
            lckconn.management_locks.list_by_scope(
                scope=scope,
                filter=kwargs.get('filter')
            )
        )

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) DNS Record Set Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 2.7.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.34.3
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.2
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

    **cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
import logging

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.dns.models  # pylint: disable=unused-import
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def create_or_update(hub, name, zone_name, resource_group, record_type, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Creates or updates a record set within a DNS zone.

    :param name: The name of the record set, relative to the name of the zone.

    :param zone_name: The name of the DNS zone (without a terminating dot).

    :param resource_group: The name of the resource group.

    :param record_type: The type of DNS record in this record set. Record sets of type SOA can be
    updated but not created (they are created when the DNS zone is created).
    Possible values include: 'A', 'AAAA', 'CAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'

    CLI Example:

    .. code-block:: bash

        azurerm.dns.record_set.create_or_update myhost myzone testgroup A
            arecords='[{ipv4_address: 10.0.0.1}]' ttl=300

    '''
    dnsconn = await hub.exec.utils.azurerm.get_client('dns', **kwargs)

    try:
        record_set_model = await hub.exec.utils.azurerm.create_object_model('dns', 'RecordSet', **kwargs)
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        record_set = dnsconn.record_sets.create_or_update(
            relative_record_set_name=name,
            zone_name=zone_name,
            resource_group_name=resource_group,
            record_type=record_type,
            parameters=record_set_model,
            if_match=kwargs.get('if_match'),
            if_none_match=kwargs.get('if_none_match')
        )
        result = record_set.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('dns', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def delete(hub, name, zone_name, resource_group, record_type, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Deletes a record set from a DNS zone. This operation cannot be undone.

    :param name: The name of the record set, relative to the name of the zone.

    :param zone_name: The name of the DNS zone (without a terminating dot).

    :param resource_group: The name of the resource group.

    :param record_type: The type of DNS record in this record set. Record sets of type SOA cannot be
    deleted (they are deleted when the DNS zone is deleted).
    Possible values include: 'A', 'AAAA', 'CAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'

    CLI Example:

    .. code-block:: bash

        azurerm.dns.record_set.delete myhost myzone testgroup A

    '''
    result = False
    dnsconn = await hub.exec.utils.azurerm.get_client('dns', **kwargs)
    try:
        record_set = dnsconn.record_sets.delete(
            relative_record_set_name=name,
            zone_name=zone_name,
            resource_group_name=resource_group,
            record_type=record_type,
            if_match=kwargs.get('if_match')
        )
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('dns', str(exc), **kwargs)

    return result


async def get(hub, name, zone_name, resource_group, record_type, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get a dictionary representing a record set's properties.

    :param name: The name of the record set, relative to the name of the zone.

    :param zone_name: The name of the DNS zone (without a terminating dot).

    :param resource_group: The name of the resource group.

    :param record_type: The type of DNS record in this record set.
    Possible values include: 'A', 'AAAA', 'CAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'

    CLI Example:

    .. code-block:: bash

        azurerm.dns.record_set.get '@' myzone testgroup SOA

    '''
    dnsconn = await hub.exec.utils.azurerm.get_client('dns', **kwargs)
    try:
        record_set = dnsconn.record_sets.get(
            relative_record_set_name=name,
            zone_name=zone_name,
            resource_group_name=resource_group,
            record_type=record_type
        )
        result = record_set.as_dict()

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('dns', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_by_type(hub, zone_name, resource_group, record_type, top=None, recordsetnamesuffix=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Lists the record sets of a specified type in a DNS zone.

    :param zone_name: The name of the DNS zone (without a terminating dot).

    :param resource_group: The name of the resource group.

    :param record_type: The type of record sets to enumerate.
    Possible values include: 'A', 'AAAA', 'CAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'

    :param top: The maximum number of record sets to return. If not specified,
    returns up to 100 record sets.

    :param recordsetnamesuffix: The suffix label of the record set name that has
    to be used to filter the record set enumerations.

    CLI Example:

    .. code-block:: bash

        azurerm.dns.record_set.list_by_type myzone testgroup SOA

    '''
    result = {}
    dnsconn = await hub.exec.utils.azurerm.get_client('dns', **kwargs)
    try:
        record_sets = await hub.exec.utils.azurerm.paged_object_to_list(
            dnsconn.record_sets.list_by_type(
                zone_name=zone_name,
                resource_group_name=resource_group,
                record_type=record_type,
                top=top,
                recordsetnamesuffix=recordsetnamesuffix
            )
        )

        for record_set in record_sets:
            result[record_set['name']] = record_set
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('dns', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_by_dns_zone(hub, zone_name, resource_group, top=None, recordsetnamesuffix=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Lists all record sets in a DNS zone.

    :param zone_name: The name of the DNS zone (without a terminating dot).

    :param resource_group: The name of the resource group.

    :param top: The maximum number of record sets to return. If not specified,
    returns up to 100 record sets.

    :param recordsetnamesuffix: The suffix label of the record set name that has
    to be used to filter the record set enumerations.

    CLI Example:

    .. code-block:: bash

        azurerm.dns.record_set.list_by_dns_zone myzone testgroup

    '''
    result = {}
    dnsconn = await hub.exec.utils.azurerm.get_client('dns', **kwargs)
    try:
        record_sets = await hub.exec.utils.azurerm.paged_object_to_list(
            dnsconn.record_sets.list_by_dns_zone(
                zone_name=zone_name,
                resource_group_name=resource_group,
                top=top,
                recordsetnamesuffix=recordsetnamesuffix
            )
        )

        for record_set in record_sets:
            result[record_set['name']] = record_set
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('dns', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Local Network Gateway Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
    to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

**cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
import logging

try:
    from six.moves import range as six_range
except ImportError:
    six_range = range

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.network.models  # pylint: disable=unused-import
    from msrestazure.tools import is_valid_resource_id, parse_resource_id
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def create_or_update(hub, name, resource_group, gateway_ip_address, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Creates or updates a local network gateway object in the specified resource group.

    :param name: The name of the local network gateway object to be created or updated.

    :param resource_group: The name of the resource group associated with the local network gateway.

    :param gateway_ip_address: IP address of the local network gateway.

    CLI Example:

    .. code-block:: bash

        azurerm.network.local_network_gateway.create_or_update test_name test_group test_ip

    '''
    if 'location' not in kwargs:
        rg_props = await hub.exec.azurerm.resource.group.get(
            resource_group, **kwargs
        )

        if 'error' in rg_props:
            log.error(
                'Unable to determine location from resource group specified.'
            )
            return False
        kwargs['location'] = rg_props['location']

    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)

    try:
        gatewaymodel = await hub.exec.utils.azurerm.create_object_model(
            'network',
            'LocalNetworkGateway',
            gateway_ip_address=gateway_ip_address,
            **kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        gateway = netconn.local_network_gateways.create_or_update(
            local_network_gateway_name=name,
            resource_group_name=resource_group,
            parameters=gatewaymodel
        )
        gateway.wait()
        gateway_result = gateway.result()
        result = gateway_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def get(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets the details of a specific local network gateway within a specified resource group.

    :param name: The name of the local network gateway.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.local_network_gateway.get test_name test_group

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        gateway = netconn.local_network_gateways.get(
            resource_group_name=resource_group,
            local_network_gateway_name=name
        )

        result = gateway.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def delete(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Deletes the specified local network gateway.

    :param name: The name of the local network gateway that will be deleted.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.local_network_gateway.delete test_name test_group

    '''
    result = False
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        gateway = netconn.local_network_gateways.delete(
            resource_group_name=resource_group,
            local_network_gateway_name=name
        )
        gateway.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)

    return result


async def list_(hub, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Lists all local network gateways within a resource group.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.local_network_gateway.list test_group

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        gateways = await hub.exec.utils.azurerm.paged_object_to_list(
            netconn.local_network_gateways.list(
                resource_group_name=resource_group
            )
        )

        for gateway in gateways:
            result[gateway['name']] = gateway
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Network Load Balancer Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
    to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

**cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
import logging

try:
    from six.moves import range as six_range
except ImportError:
    six_range = range

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.network.models  # pylint: disable=unused-import
    from msrestazure.tools import is_valid_resource_id, parse_resource_id
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def list_all(hub, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all load balancers within a subscription.

    CLI Example:

    .. code-block:: bash

        azurerm.network.load_balancer.list_all

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        load_balancers = await hub.exec.utils.azurerm.paged_object_to_list(netconn.load_balancers.list_all())

        for load_balancer in load_balancers:
            result[load_balancer['name']] = load_balancer
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_(hub, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all load balancers within a resource group.

    :param resource_group: The resource group name to list load balancers
        within.

    CLI Example:

    .. code-block:: bash

        azurerm.network.load_balancer.list testgroup

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        load_balancers = await hub.exec.utils.azurerm.paged_object_to_list(
            netconn.load_balancers.list(
                resource_group_name=resource_group
            )
        )

        for load_balancer in load_balancers:
            result[load_balancer['name']] = load_balancer
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def get(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get details about a specific load balancer.

    :param name: The name of the load balancer to query.

    :param resource_group: The resource group name assigned to the
        load balancer.

    CLI Example:

    .. code-block:: bash

        azurerm.network.load_balancer.get testlb testgroup

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        load_balancer = netconn.load_balancers.get(
            load_balancer_name=name,
            resource_group_name=resource_group
        )
        result = load_balancer.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def create_or_update(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Create or update a load balancer within a specified resource group.

    :param name: The name of the load balancer to create.

    :param resource_group: The resource group name assigned to the
        load balancer.

    CLI Example:

    .. code-block:: bash

        azurerm.network.load_balancer.create_or_update testlb testgroup

    '''
    if 'location' not in kwargs:
        rg_props = await hub.exec.azurerm.resource.group.get(
            resource_group, **kwargs
        )

        if 'error' in rg_props:
            log.error(
                'Unable to determine location from resource group specified.'
            )
            return False
        kwargs['location'] = rg_props['location']

    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)

    if isinstance(kwargs.get('frontend_ip_configurations'), list):
        for idx in six_range(0, len(kwargs['frontend_ip_configurations'])):
            # Use Public IP Address name to link to the ID of an existing Public IP
            if 'public_ip_address' in kwargs['frontend_ip_configurations'][idx]:
                pub_ip = public_ip_address_get(
                    name=kwargs['frontend_ip_configurations'][idx]['public_ip_address'],
                    resource_group=resource_group,
                    **kwargs
                )
                if 'error' not in pub_ip:
                    kwargs['frontend_ip_configurations'][idx]['public_ip_address'] = {'id': str(pub_ip['id'])}
            # Use Subnet name to link to the ID of an existing Subnet
            elif 'subnet' in kwargs['frontend_ip_configurations'][idx]:
                vnets = virtual_networks_list(
                    resource_group=resource_group,
                    **kwargs
                )
                if 'error' not in vnets:
                    for vnet in vnets:
                        subnets = subnets_list(
                            virtual_network=vnet,
                            resource_group=resource_group,
                            **kwargs
                        )
                        if kwargs['frontend_ip_configurations'][idx]['subnet'] in subnets:
                            kwargs['frontend_ip_configurations'][idx]['subnet'] = {
                                'id': str(subnets[kwargs['frontend_ip_configurations'][idx]['subnet']]['id'])
                            }
                            break

    id_url = '/subscriptions/{0}/resourceGroups/{1}/providers/Microsoft.Network/loadBalancers/{2}/{3}/{4}'

    if isinstance(kwargs.get('load_balancing_rules'), list):
        for idx in six_range(0, len(kwargs['load_balancing_rules'])):
            # Link to sub-objects which might be created at the same time as the load balancer
            if 'frontend_ip_configuration' in kwargs['load_balancing_rules'][idx]:
                kwargs['load_balancing_rules'][idx]['frontend_ip_configuration'] = {
                    'id': id_url.format(
                        kwargs.get('subscription_id'),
                        resource_group,
                        name,
                        'frontendIPConfigurations',
                        kwargs['load_balancing_rules'][idx]['frontend_ip_configuration']
                    )
                }
            if 'backend_address_pool' in kwargs['load_balancing_rules'][idx]:
                kwargs['load_balancing_rules'][idx]['backend_address_pool'] = {
                    'id': id_url.format(
                        kwargs.get('subscription_id'),
                        resource_group,
                        name,
                        'backendAddressPools',
                        kwargs['load_balancing_rules'][idx]['backend_address_pool']
                    )
                }
            if 'probe' in kwargs['load_balancing_rules'][idx]:
                kwargs['load_balancing_rules'][idx]['probe'] = {
                    'id': id_url.format(
                        kwargs.get('subscription_id'),
                        resource_group,
                        name,
                        'probes',
                        kwargs['load_balancing_rules'][idx]['probe']
                    )
                }

    if isinstance(kwargs.get('inbound_nat_rules'), list):
        for idx in six_range(0, len(kwargs['inbound_nat_rules'])):
            # Link to sub-objects which might be created at the same time as the load balancer
            if 'frontend_ip_configuration' in kwargs['inbound_nat_rules'][idx]:
                kwargs['inbound_nat_rules'][idx]['frontend_ip_configuration'] = {
                    'id': id_url.format(
                        kwargs.get('subscription_id'),
                        resource_group,
                        name,
                        'frontendIPConfigurations',
                        kwargs['inbound_nat_rules'][idx]['frontend_ip_configuration']
                    )
                }

    if isinstance(kwargs.get('inbound_nat_pools'), list):
        for idx in six_range(0, len(kwargs['inbound_nat_pools'])):
            # Link to sub-objects which might be created at the same time as the load balancer
            if 'frontend_ip_configuration' in kwargs['inbound_nat_pools'][idx]:
                kwargs['inbound_nat_pools'][idx]['frontend_ip_configuration'] = {
                    'id': id_url.format(
                        kwargs.get('subscription_id'),
                        resource_group,
                        name,
                        'frontendIPConfigurations',
                        kwargs['inbound_nat_pools'][idx]['frontend_ip_configuration']
                    )
                }

    if isinstance(kwargs.get('outbound_nat_rules'), list):
        for idx in six_range(0, len(kwargs['outbound_nat_rules'])):
            # Link to sub-objects which might be created at the same time as the load balancer
            if 'frontend_ip_configuration' in kwargs['outbound_nat_rules'][idx]:
                kwargs['outbound_nat_rules'][idx]['frontend_ip_configuration'] = {
                    'id': id_url.format(
                        kwargs.get('subscription_id'),
                        resource_group,
                        name,
                        'frontendIPConfigurations',
                        kwargs['outbound_nat_rules'][idx]['frontend_ip_configuration']
                    )
                }
            if 'backend_address_pool' in kwargs['outbound_nat_rules'][idx]:
                kwargs['outbound_nat_rules'][idx]['backend_address_pool'] = {
                    'id': id_url.format(
                        kwargs.get('subscription_id'),
                        resource_group,
                        name,
                        'backendAddressPools',
                        kwargs['outbound_nat_rules'][idx]['backend_address_pool']
                    )
                }

    try:
        lbmodel = await hub.exec.utils.azurerm.create_object_model('network', 'LoadBalancer', **kwargs)
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        load_balancer = netconn.load_balancers.create_or_update(
            resource_group_name=resource_group,
            load_balancer_name=name,
            parameters=lbmodel
        )
        load_balancer.wait()
        lb_result = load_balancer.result()
        result = lb_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def delete(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Delete a load balancer.

    :param name: The name of the load balancer to delete.

    :param resource_group: The resource group name assigned to the
        load balancer.

    CLI Example:

    .. code-block:: bash

        azurerm.network.load_balancer.delete testlb testgroup

    '''
    result = False
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        load_balancer = netconn.load_balancers.delete(
            load_balancer_name=name,
            resource_group_name=resource_group
        )
        load_balancer.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)

    return result



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Resource Group State Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 2.7.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.34.3
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.2
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as a dictionary of
    keyword arguments to the ``connection_auth`` parameter in order to work properly. Since the authentication
    parameters are sensitive, it's recommended to pass them to the states via pillar.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

    **cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud. Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

    Example configuration for Azure Resource Manager authentication:

    .. code-block:: yaml

        azurerm:
            user_pass_auth:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                username: fletch
                password: 123pass
            mysubscription:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                tenant: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                client_id: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                secret: XXXXXXXXXXXXXXXXXXXXXXXX
                cloud_environment: AZURE_PUBLIC_CLOUD

    Example states using Azure Resource Manager authentication:

    .. code-block:: jinja

        Ensure resource group exists:
            azurerm.resource.group.present:
                - name: my_rg
                - location: westus
                - tags:
                    how_awesome: very
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}

        Ensure resource group is absent:
            azurerm.resource.group.absent:
                - name: other_rg
                - connection_auth: {{ profile }}

'''

# Import Python libs
from __future__ import absolute_import
import json
import logging

log = logging.getLogger(__name__)


async def present(hub, ctx, name, location, managed_by=None, tags=None, connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a resource group exists.

    :param name:
        Name of the resource group.

    :param location:
        The Azure location in which to create the resource group. This value cannot be updated once
        the resource group is created.

    :param managed_by:
        The ID of the resource that manages this resource group. This value cannot be updated once
        the resource group is created.

    :param tags:
        A dictionary of strings can be passed as tag metadata to the resource group object.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure resource group exists:
            azurerm.resource.group.present:
                - name: group1
                - location: eastus
                - tags:
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}

    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    group = {}

    present = await hub.exec.azurerm.resource.group.check_existence(name, **connection_auth)

    if present:
        group = await hub.exec.azurerm.resource.group.get(name, **connection_auth)
        ret['changes'] = await hub.exec.utils.dictdiffer.deep_diff(group.get('tags', {}), tags or {})

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'Resource group {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['comment'] = 'Resource group {0} tags would be updated.'.format(name)
            ret['result'] = None
            ret['changes'] = {
                'old': group.get('tags', {}),
                'new': tags
            }
            return ret

    elif ctx['test']:
        ret['comment'] = 'Resource group {0} would be created.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': {},
            'new': {
                'name': name,
                'location': location,
                'managed_by': managed_by,
                'tags': tags,
            }
        }
        return ret

    group_kwargs = kwargs.copy()
    group_kwargs.update(connection_auth)

    group = await hub.exec.azurerm.resource.group.create_or_update(
        name,
        location,
        managed_by=managed_by,
        tags=tags,
        **group_kwargs
    )
    present = await hub.exec.azurerm.resource.group.check_existence(name, **connection_auth)

    if present:
        ret['result'] = True
        ret['comment'] = 'Resource group {0} has been created.'.format(name)
        ret['changes'] = {
            'old': {},
            'new': group
        }
        return ret

    ret['comment'] = 'Failed to create resource group {0}! ({1})'.format(name, group.get('error'))
    return ret


async def absent(hub, ctx, name, connection_auth=None):
    '''
    .. versionadded:: 1.0.0

    Ensure a resource group does not exist in the current subscription.

    :param name:
        Name of the resource group.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    group = {}

    present = await hub.exec.azurerm.resource.group.check_existence(name, **connection_auth)

    if not present:
        ret['result'] = True
        ret['comment'] = 'Resource group {0} is already absent.'.format(name)
        return ret

    elif ctx['test']:
        group = await hub.exec.azurerm.resource.group.get(name, **connection_auth)

        ret['comment'] = 'Resource group {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': group,
            'new': {},
        }
        return ret

    group = await hub.exec.azurerm.resource.group.get(name, **connection_auth)
    deleted = await hub.exec.azurerm.resource.group.delete(name, **connection_auth)

    if deleted:
        present = False
    else:
        present = await hub.exec.azurerm.resource.group.check_existence(name, **connection_auth)

    if not present:
        ret['result'] = True
        ret['comment'] = 'Resource group {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': group,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete resource group {0}!'.format(name)
    return ret



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Virtual Network Peering Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
    to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

**cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
import logging

try:
    from six.moves import range as six_range
except ImportError:
    six_range = range

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.network.models  # pylint: disable=unused-import
    from msrestazure.tools import is_valid_resource_id, parse_resource_id
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def list_(hub, virtual_network, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all peerings associated with a virtual network.

    :param virtual_network: The virtual network name for which to list peerings.

    :param resource_group: The resource group name for the virtual network.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_peering.list testnet testgroup

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        peerings = await hub.exec.utils.azurerm.paged_object_to_list(
            netconn.virtual_network_peerings.list(
                resource_group_name=resource_group,
                virtual_network_name=virtual_network
            )
        )

        for peering in peerings:
            result[peering['name']] = peering
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def delete(hub, name, virtual_network, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Delete a virtual network peering object.

    :param name: The name of the virtual network peering object to delete.

    :param virtual_network: The virtual network name containing the
        peering object.

    :param resource_group: The resource group name assigned to the
        virtual network.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_peering.delete peer1 testnet testgroup

    '''
    result = False
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        peering = netconn.virtual_network_peerings.delete(
            resource_group_name=resource_group,
            virtual_network_name=virtual_network,
            virtual_network_peering_name=name
        )
        peering.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)

    return result


async def get(hub, name, virtual_network, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get details about a specific virtual network peering object.

    :param name: The name of the virtual network peering to query.

    :param virtual_network: The virtual network name containing the
        peering object.

    :param resource_group: The resource group name assigned to the
        virtual network.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_peering.get peer1 testnet testgroup

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        peering = netconn.virtual_network_peerings.get(
            resource_group_name=resource_group,
            virtual_network_name=virtual_network,
            virtual_network_peering_name=name
        )

        result = peering.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def create_or_update(hub, name, remote_virtual_network, virtual_network, resource_group, remote_vnet_group=None,
                           **kwargs):
    '''
    .. versionadded:: 1.0.0

    Create or update a virtual network peering object.

    :param name: The name assigned to the peering object being created or updated.

    :param remote_virtual_network: A valid name of a virtual network with which to peer.

    :param remote_vnet_group: The resource group of the remote virtual network. Defaults to
        the same resource group as the "local" virtual network.

    :param virtual_network: The virtual network name containing the
        peering object.

    :param resource_group: The resource group name assigned to the
        virtual network.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_peering.create_or_update peer1 \
                  remotenet testnet testgroup remote_vnet_group=remotegroup

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)

    # Use Remote Virtual Network name to link to the ID of an existing object
    remote_vnet = virtual_network_get(
        name=remote_virtual_network,
        resource_group=(remote_vnet_group or resource_group),
        **kwargs
    )
    if 'error' not in remote_vnet:
        remote_virtual_network = {'id': str(remote_vnet['id'])}

    try:
        peermodel = await hub.exec.utils.azurerm.create_object_model(
            'network',
            'VirtualNetworkPeering',
            remote_virtual_network=remote_virtual_network,
            **kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        peering = netconn.virtual_network_peerings.create_or_update(
            resource_group_name=resource_group,
            virtual_network_name=virtual_network,
            virtual_network_peering_name=name,
            virtual_network_peering_parameters=peermodel
        )
        peering.wait()
        peer_result = peering.result()
        result = peer_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Network Security Group Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
    to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

**cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
import logging

try:
    from six.moves import range as six_range
except ImportError:
    six_range = range

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.network.models  # pylint: disable=unused-import
    from msrestazure.tools import is_valid_resource_id, parse_resource_id
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def default_security_rule_get(hub, name, security_group, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get details about a default security rule within a security group.

    :param name: The name of the security rule to query.

    :param security_group: The network security group containing the
        security rule.

    :param resource_group: The resource group name assigned to the
        network security group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.network_security_group.default_security_rule_get DenyAllOutBound testnsg testgroup

    '''
    result = {}

    default_rules = default_security_rules_list(
        security_group=security_group,
        resource_group=resource_group,
        **kwargs
    )

    if isinstance(default_rules, dict) and 'error' in default_rules:
        return default_rules

    try:
        for default_rule in default_rules:
            if default_rule['name'] == name:
                result = default_rule
        if not result:
            result = {
                'error': 'Unable to find {0} in {1}!'.format(name, security_group)
            }
    except KeyError as exc:
        log.error(
            'Unable to find {0} in {1}!'.format(name, security_group)
        )
        result = {'error': str(exc)}

    return result


async def default_security_rules_list(hub, security_group, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List default security rules within a security group.

    :param security_group: The network security group to query.

    :param resource_group: The resource group name assigned to the
        network security group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.network_security_group.default_security_rules_list testnsg testgroup

    '''
    result = {}

    secgroup = network_security_group_get(
        security_group=security_group,
        resource_group=resource_group,
        **kwargs
    )

    if 'error' in secgroup:
        return secgroup

    try:
        result = secgroup['default_security_rules']
    except KeyError as exc:
        log.error(
            'No default security rules found for {0}!'.format(security_group)
        )
        result = {'error': str(exc)}

    return result


async def security_rules_list(hub, security_group, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List security rules within a network security group.

    :param security_group: The network security group to query.

    :param resource_group: The resource group name assigned to the
        network security group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.network_security_group.security_rules_list testnsg testgroup

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        secrules = netconn.security_rules.list(
            network_security_group_name=security_group,
            resource_group_name=resource_group
        )
        result = await hub.exec.utils.azurerm.paged_object_to_list(secrules)
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def security_rule_create_or_update(hub, name, access, direction, priority, protocol, security_group,
                                         resource_group, source_address_prefix=None, destination_address_prefix=None,
                                         source_port_range=None, destination_port_range=None,
                                         source_address_prefixes=None, destination_address_prefixes=None,
                                         source_port_ranges=None, destination_port_ranges=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Create or update a security rule within a specified network security group.

    :param name: The name of the security rule to create.

    :param access:
        'allow' or 'deny'

    :param direction:
        'inbound' or 'outbound'

    :param priority:
        Integer between 100 and 4096 used for ordering rule application.

    :param protocol:
        'tcp', 'udp', or '*'

    :param destination_address_prefix:
        The CIDR or destination IP range. Asterix '*' can also be used to match all destination IPs.
        Default tags such as 'VirtualNetwork', 'AzureLoadBalancer' and 'Internet' can also be used.
        If this is an ingress rule, specifies where network traffic originates from.

    :param destination_port_range:
        The destination port or range. Integer or range between 0 and 65535. Asterix '*'
        can also be used to match all ports.

    :param source_address_prefix:
        The CIDR or source IP range. Asterix '*' can also be used to match all source IPs.
        Default tags such as 'VirtualNetwork', 'AzureLoadBalancer' and 'Internet' can also be used.
        If this is an ingress rule, specifies where network traffic originates from.

    :param source_port_range:
        The source port or range. Integer or range between 0 and 65535. Asterix '*'
        can also be used to match all ports.

    :param destination_address_prefixes:
        A list of destination_address_prefix values. This parameter overrides destination_address_prefix
        and will cause any value entered there to be ignored.

    :param destination_port_ranges:
        A list of destination_port_range values. This parameter overrides destination_port_range
        and will cause any value entered there to be ignored.

    :param source_address_prefixes:
        A list of source_address_prefix values. This parameter overrides source_address_prefix
        and will cause any value entered there to be ignored.

    :param source_port_ranges:
        A list of source_port_range values. This parameter overrides source_port_range
        and will cause any value entered there to be ignored.

    :param security_group: The network security group containing the
        security rule.

    :param resource_group: The resource group name assigned to the
        network security group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.network_security_group.security_rule_create_or_update testrule1 allow outbound 101 tcp \
                  testnsg testgroup source_address_prefix='*' destination_address_prefix=internet \
                  source_port_range='*' destination_port_range='1-1024'

    '''
    exclusive_params = [
        ('source_port_ranges', 'source_port_range'),
        ('source_address_prefixes', 'source_address_prefix'),
        ('destination_port_ranges', 'destination_port_range'),
        ('destination_address_prefixes', 'destination_address_prefix'),
    ]

    for params in exclusive_params:
        # pylint: disable=eval-used
        if not eval(params[0]) and not eval(params[1]):
            log.error(
                'Either the {0} or {1} parameter must be provided!'.format(params[0], params[1])
            )
            return False
        # pylint: disable=eval-used
        if eval(params[0]):
            # pylint: disable=exec-used
            exec('{0} = None'.format(params[1]))

    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)

    try:
        rulemodel = await hub.exec.utils.azurerm.create_object_model(
            'network',
            'SecurityRule',
            name=name,
            access=access,
            direction=direction,
            priority=priority,
            protocol=protocol,
            source_port_ranges=source_port_ranges,
            source_port_range=source_port_range,
            source_address_prefixes=source_address_prefixes,
            source_address_prefix=source_address_prefix,
            destination_port_ranges=destination_port_ranges,
            destination_port_range=destination_port_range,
            destination_address_prefixes=destination_address_prefixes,
            destination_address_prefix=destination_address_prefix,
            **kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        secrule = netconn.security_rules.create_or_update(
            resource_group_name=resource_group,
            network_security_group_name=security_group,
            security_rule_name=name,
            security_rule_parameters=rulemodel
        )
        secrule.wait()
        secrule_result = secrule.result()
        result = secrule_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def security_rule_delete(hub, security_rule, security_group, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Delete a security rule within a specified security group.

    :param name: The name of the security rule to delete.

    :param security_group: The network security group containing the
        security rule.

    :param resource_group: The resource group name assigned to the
        network security group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.network_security_group.security_rule_delete testrule1 testnsg testgroup

    '''
    result = False
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        secrule = netconn.security_rules.delete(
            network_security_group_name=security_group,
            resource_group_name=resource_group,
            security_rule_name=security_rule
        )
        secrule.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)

    return result


async def security_rule_get(hub, security_rule, security_group, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get a security rule within a specified network security group.

    :param name: The name of the security rule to query.

    :param security_group: The network security group containing the
        security rule.

    :param resource_group: The resource group name assigned to the
        network security group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.network_security_group.security_rule_get testrule1 testnsg testgroup

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        secrule = netconn.security_rules.get(
            network_security_group_name=security_group,
            resource_group_name=resource_group,
            security_rule_name=security_rule
        )
        result = secrule.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def create_or_update(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Create or update a network security group.

    :param name: The name of the network security group to create.

    :param resource_group: The resource group name assigned to the
        network security group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.network_security_group.create_or_update testnsg testgroup

    '''
    if 'location' not in kwargs:
        rg_props = await hub.exec.azurerm.resource.group.get(
            resource_group, **kwargs
        )

        if 'error' in rg_props:
            log.error(
                'Unable to determine location from resource group specified.'
            )
            return False
        kwargs['location'] = rg_props['location']

    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)

    try:
        secgroupmodel = await hub.exec.utils.azurerm.create_object_model('network', 'NetworkSecurityGroup', **kwargs)
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        secgroup = netconn.network_security_groups.create_or_update(
            resource_group_name=resource_group,
            network_security_group_name=name,
            parameters=secgroupmodel
        )
        secgroup.wait()
        secgroup_result = secgroup.result()
        result = secgroup_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def delete(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Delete a network security group within a resource group.

    :param name: The name of the network security group to delete.

    :param resource_group: The resource group name assigned to the
        network security group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.network_security_group.delete testnsg testgroup

    '''
    result = False
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        secgroup = netconn.network_security_groups.delete(
            resource_group_name=resource_group,
            network_security_group_name=name
        )
        secgroup.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)

    return result


async def get(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get details about a network security group within a resource group.

    :param name: The name of the network security group to query.

    :param resource_group: The resource group name assigned to the
        network security group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.network_security_group.get testnsg testgroup

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        secgroup = netconn.network_security_groups.get(
            resource_group_name=resource_group,
            network_security_group_name=name
        )
        result = secgroup.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_(hub, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all network security groups within a resource group.

    :param resource_group: The resource group name to list network security \
        groups within.

    CLI Example:

    .. code-block:: bash

        azurerm.network.network_security_groups.list testgroup

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        secgroups = await hub.exec.utils.azurerm.paged_object_to_list(
            netconn.network_security_groups.list(
                resource_group_name=resource_group
            )
        )
        for secgroup in secgroups:
            result[secgroup['name']] = secgroup
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_all(hub, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all network security groups within a subscription.

    CLI Example:

    .. code-block:: bash

        azurerm.network.network_security_groups.list_all

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        secgroups = await hub.exec.utils.azurerm.paged_object_to_list(
            netconn.network_security_groups.list_all()
        )
        for secgroup in secgroups:
            result[secgroup['name']] = secgroup
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Virtual Network State Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as a dictionary of
    keyword arguments to the ``connection_auth`` parameter in order to work properly. Since the authentication
    parameters are sensitive, it's recommended to pass them to the states via pillar.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

    **cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud. Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

    Example Pillar for Azure Resource Manager authentication:

    .. code-block:: yaml

        azurerm:
            user_pass_auth:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                username: fletch
                password: 123pass
            mysubscription:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                tenant: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                client_id: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                secret: XXXXXXXXXXXXXXXXXXXXXXXX
                cloud_environment: AZURE_PUBLIC_CLOUD

    Example states using Azure Resource Manager authentication:

    .. code-block:: jinja

        Ensure virtual network exists:
            azurerm.network.virtual_network.present:
                - name: my_vnet
                - resource_group: my_rg
                - address_prefixes:
                    - '10.0.0.0/8'
                    - '192.168.0.0/16'
                - dns_servers:
                    - '8.8.8.8'
                - tags:
                    how_awesome: very
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}

        Ensure virtual network is absent:
            azurerm.network.virtual_network.absent:
                - name: other_vnet
                - resource_group: my_rg
                - connection_auth: {{ profile }}

'''

# Python libs
from __future__ import absolute_import
import logging
import re

try:
    from six.moves import range as six_range
except ImportError:
    six_range = range

log = logging.getLogger(__name__)


async def present(hub, ctx, name, address_prefixes, resource_group, dns_servers=None, tags=None, connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a virtual network exists.

    :param name:
        Name of the virtual network.

    :param resource_group:
        The resource group assigned to the virtual network.

    :param address_prefixes:
        A list of CIDR blocks which can be used by subnets within the virtual network.

    :param dns_servers:
        A list of DNS server addresses.

    :param tags:
        A dictionary of strings can be passed as tag metadata to the virtual network object.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure virtual network exists:
            azurerm.network.virtual_network.present:
                - name: vnet1
                - resource_group: group1
                - address_prefixes:
                    - '10.0.0.0/8'
                    - '192.168.0.0/16'
                - dns_servers:
                    - '8.8.8.8'
                - tags:
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}
                - require:
                  - azurearm_resource: Ensure resource group exists

    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    vnet = await hub.exec.azurerm.network.virtual_network.get(
        name,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' not in vnet:
        tag_changes = await hub.exec.utils.dictdiffer.deep_diff(vnet.get('tags', {}), tags or {})
        if tag_changes:
            ret['changes']['tags'] = tag_changes

        dns_changes = set(dns_servers or []).symmetric_difference(
            set(vnet.get('dhcp_options', {}).get('dns_servers', [])))
        if dns_changes:
            ret['changes']['dns_servers'] = {
                'old': vnet.get('dhcp_options', {}).get('dns_servers', []),
                'new': dns_servers,
            }

        addr_changes = set(address_prefixes or []).symmetric_difference(
            set(vnet.get('address_space', {}).get('address_prefixes', [])))
        if addr_changes:
            ret['changes']['address_space'] = {
                'address_prefixes': {
                    'old': vnet.get('address_space', {}).get('address_prefixes', []),
                    'new': address_prefixes,
                }
            }

        if kwargs.get('enable_ddos_protection', False) != vnet.get('enable_ddos_protection'):
            ret['changes']['enable_ddos_protection'] = {
                'old': vnet.get('enable_ddos_protection'),
                'new': kwargs.get('enable_ddos_protection')
            }

        if kwargs.get('enable_vm_protection', False) != vnet.get('enable_vm_protection'):
            ret['changes']['enable_vm_protection'] = {
                'old': vnet.get('enable_vm_protection'),
                'new': kwargs.get('enable_vm_protection')
            }

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'Virtual network {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['result'] = None
            ret['comment'] = 'Virtual network {0} would be updated.'.format(name)
            return ret

    else:
        ret['changes'] = {
            'old': {},
            'new': {
                'name': name,
                'resource_group': resource_group,
                'address_space': {'address_prefixes': address_prefixes},
                'dhcp_options': {'dns_servers': dns_servers},
                'enable_ddos_protection': kwargs.get('enable_ddos_protection', False),
                'enable_vm_protection': kwargs.get('enable_vm_protection', False),
                'tags': tags,
            }
        }

    if ctx['test']:
        ret['comment'] = 'Virtual network {0} would be created.'.format(name)
        ret['result'] = None
        return ret

    vnet_kwargs = kwargs.copy()
    vnet_kwargs.update(connection_auth)

    vnet = await hub.exec.azurerm.network.virtual_network.create_or_update(
        name=name,
        resource_group=resource_group,
        address_prefixes=address_prefixes,
        dns_servers=dns_servers,
        tags=tags,
        **vnet_kwargs
    )

    if 'error' not in vnet:
        ret['result'] = True
        ret['comment'] = 'Virtual network {0} has been created.'.format(name)
        return ret

    ret['comment'] = 'Failed to create virtual network {0}! ({1})'.format(name, vnet.get('error'))
    return ret


async def absent(hub, ctx, name, resource_group, connection_auth=None):
    '''
    .. versionadded:: 1.0.0

    Ensure a virtual network does not exist in the resource group.

    :param name:
        Name of the virtual network.

    :param resource_group:
        The resource group assigned to the virtual network.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    vnet = await hub.exec.azurerm.network.virtual_network.get(
        name,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' in vnet:
        ret['result'] = True
        ret['comment'] = 'Virtual network {0} was not found.'.format(name)
        return ret

    elif ctx['test']:
        ret['comment'] = 'Virtual network {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': vnet,
            'new': {},
        }
        return ret

    deleted = await hub.exec.azurerm.network.virtual_network.delete(name, resource_group, **connection_auth)

    if deleted:
        ret['result'] = True
        ret['comment'] = 'Virtual network {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': vnet,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete virtual network {0}!'.format(name)
    return ret


async def subnet_present(hub, ctx, name, address_prefix, virtual_network, resource_group, security_group=None, route_table=None,
                   connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a subnet exists.

    :param name:
        Name of the subnet.

    :param address_prefix:
        A CIDR block used by the subnet within the virtual network.

    :param virtual_network:
        Name of the existing virtual network to contain the subnet.

    :param resource_group:
        The resource group assigned to the virtual network.

    :param security_group:
        The name of the existing network security group to assign to the subnet.

    :param route_table:
        The name of the existing route table to assign to the subnet.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure subnet exists:
            azurerm.network.virtual_network.subnet_present:
                - name: vnet1_sn1
                - virtual_network: vnet1
                - resource_group: group1
                - address_prefix: '192.168.1.0/24'
                - security_group: nsg1
                - route_table: rt1
                - connection_auth: {{ profile }}
                - require:
                  - azurearm_network: Ensure virtual network exists
                  - azurearm_network: Ensure network security group exists
                  - azurearm_network: Ensure route table exists

    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    snet = await hub.exec.azurerm.network.virtual_network.subnet_get(
        name,
        virtual_network,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' not in snet:
        if address_prefix != snet.get('address_prefix'):
            ret['changes']['address_prefix'] = {
                'old': snet.get('address_prefix'),
                'new': address_prefix
            }

        nsg_name = None
        if snet.get('network_security_group'):
            nsg_name = snet['network_security_group']['id'].split('/')[-1]

        if security_group and (security_group != nsg_name):
            ret['changes']['network_security_group'] = {
                'old': nsg_name,
                'new': security_group
            }

        rttbl_name = None
        if snet.get('route_table'):
            rttbl_name = snet['route_table']['id'].split('/')[-1]

        if route_table and (route_table != rttbl_name):
            ret['changes']['route_table'] = {
                'old': rttbl_name,
                'new': route_table
            }

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'Subnet {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['result'] = None
            ret['comment'] = 'Subnet {0} would be updated.'.format(name)
            return ret

    else:
        ret['changes'] = {
            'old': {},
            'new': {
                'name': name,
                'address_prefix': address_prefix,
                'network_security_group': security_group,
                'route_table': route_table
            }
        }

    if ctx['test']:
        ret['comment'] = 'Subnet {0} would be created.'.format(name)
        ret['result'] = None
        return ret

    snet_kwargs = kwargs.copy()
    snet_kwargs.update(connection_auth)

    snet = await hub.exec.azurerm.network.virtual_network.subnet_create_or_update(
        name=name,
        virtual_network=virtual_network,
        resource_group=resource_group,
        address_prefix=address_prefix,
        network_security_group=security_group,
        route_table=route_table,
        **snet_kwargs
    )

    if 'error' not in snet:
        ret['result'] = True
        ret['comment'] = 'Subnet {0} has been created.'.format(name)
        return ret

    ret['comment'] = 'Failed to create subnet {0}! ({1})'.format(name, snet.get('error'))
    return ret


async def subnet_absent(hub, ctx, name, virtual_network, resource_group, connection_auth=None):
    '''
    .. versionadded:: 1.0.0

    Ensure a virtual network does not exist in the virtual network.

    :param name:
        Name of the subnet.

    :param virtual_network:
        Name of the existing virtual network containing the subnet.

    :param resource_group:
        The resource group assigned to the virtual network.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    snet = await hub.exec.azurerm.network.virtual_network.subnet_get(
        name,
        virtual_network,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' in snet:
        ret['result'] = True
        ret['comment'] = 'Subnet {0} was not found.'.format(name)
        return ret

    elif ctx['test']:
        ret['comment'] = 'Subnet {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': snet,
            'new': {},
        }
        return ret

    deleted = await hub.exec.azurerm.network.virtual_network.subnet_delete(
        name,
        virtual_network,
        resource_group,
        **connection_auth
    )

    if deleted:
        ret['result'] = True
        ret['comment'] = 'Subnet {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': snet,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete subnet {0}!'.format(name)
    return ret



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Virtual Network Peering State Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as a dictionary of
    keyword arguments to the ``connection_auth`` parameter in order to work properly. Since the authentication
    parameters are sensitive, it's recommended to pass them to the states via pillar.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

    **cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud. Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

    Example Pillar for Azure Resource Manager authentication:

    .. code-block:: yaml

        azurerm:
            user_pass_auth:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                username: fletch
                password: 123pass
            mysubscription:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                tenant: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                client_id: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                secret: XXXXXXXXXXXXXXXXXXXXXXXX
                cloud_environment: AZURE_PUBLIC_CLOUD

    Example states using Azure Resource Manager authentication:

    .. code-block:: jinja

        Ensure virtual network exists:
            azurerm.network.virtual_network.present:
                - name: my_vnet
                - resource_group: my_rg
                - address_prefixes:
                    - '10.0.0.0/8'
                    - '192.168.0.0/16'
                - dns_servers:
                    - '8.8.8.8'
                - tags:
                    how_awesome: very
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}

        Ensure virtual network is absent:
            azurerm.network.virtual_network.absent:
                - name: other_vnet
                - resource_group: my_rg
                - connection_auth: {{ profile }}

'''

# Python libs
from __future__ import absolute_import
import logging
import re

try:
    from six.moves import range as six_range
except ImportError:
    six_range = range

log = logging.getLogger(__name__)


async def present(hub, ctx, name, remote_virtual_network, virtual_network, resource_group, remote_vnet_group=None,
            allow_virtual_network_access=True, allow_forwarded_traffic=False, allow_gateway_transit=False,
            use_remote_gateways=False, connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a virtual network peering object exists.

    :param name:
        Name of the peering object.

    :param remote_virtual_network:
        The name of the remote virtual network.

    :param remote_vnet_group:
        The resource group of the remote virtual network. Defaults to the same resource group
        as the local virtual network.

    :param virtual_network:
        Name of the existing virtual network to contain the peering object.

    :param resource_group:
        The resource group assigned to the local virtual network.

    :param allow_virtual_network_access:
        Whether the VMs in the local virtual network space would be able to access
        the VMs in remote virtual network space.

    :param allow_forwarded_traffic:
        Whether the forwarded traffic from the VMs in the local virtual network will
        be allowed/disallowed in remote virtual network.

    :param allow_gateway_transit:
        If gateway links can be used in remote virtual networking to link to this virtual network.

    :param use_remote_gateways:
        If remote gateways can be used on this virtual network. If the flag is set to true, and allow_gateway_transit
        on remote peering is also true, virtual network will use gateways of remote virtual network for transit.
        Only one peering can have this flag set to true. This flag cannot be set if virtual network already has a
        gateway.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure virtual network peering exists:
            azurerm.network.virtual_network_peering.present:
                - name: vnet1_to_vnet2
                - virtual_network: vnet1
                - resource_group: group1
                - remote_virtual_network: vnet2
                - remote_vnet_group: group2
                - allow_virtual_network_access: True
                - allow_forwarded_traffic: False
                - allow_gateway_transit: False
                - use_remote_gateways: False
                - connection_auth: {{ profile }}
                - require:
                  - azurearm_network: Ensure virtual network exists
                  - azurearm_network: Ensure remote virtual network exists

    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    peering = await hub.exec.azurerm.network.virtual_network_peering.get(
        name,
        virtual_network,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' not in peering:
        remote_vnet = None
        if peering.get('remote_virtual_network', {}).get('id'):
            remote_vnet = peering['remote_virtual_network']['id'].split('/')[-1]

        if remote_virtual_network != remote_vnet:
            ret['changes']['remote_virtual_network'] = {
                'old': remote_vnet,
                'new': remote_virtual_network
            }

        for bool_opt in ['use_remote_gateways', 'allow_forwarded_traffic', 'allow_virtual_network_access', 'allow_gateway_transit']:
            if locals()[bool_opt] != peering.get(bool_opt):
                ret['changes'][bool_opt] = {
                    'old': peering.get(bool_opt),
                    'new': locals()[bool_opt]
                }

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'Peering object {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['result'] = None
            ret['comment'] = 'Peering object {0} would be updated.'.format(name)
            return ret

    else:
        ret['changes'] = {
            'old': {},
            'new': {
                'name': name,
                'remote_virtual_network': remote_virtual_network,
                'remote_vnet_group': (remote_vnet_group or resource_group),
                'use_remote_gateways': use_remote_gateways,
                'allow_forwarded_traffic': allow_forwarded_traffic,
                'allow_virtual_network_access': allow_virtual_network_access,
                'allow_gateway_transit': allow_gateway_transit
            }
        }

    if ctx['test']:
        ret['comment'] = 'Subnet {0} would be created.'.format(name)
        ret['result'] = None
        return ret

    peering_kwargs = kwargs.copy()
    peering_kwargs.update(connection_auth)

    peering = await hub.exec.azurerm.network.virtual_network_peering.create_or_update(
        name=name,
        remote_virtual_network=remote_virtual_network,
        remote_vnet_group=remote_vnet_group,
        virtual_network=virtual_network,
        resource_group=resource_group,
        use_remote_gateways=use_remote_gateways,
        allow_forwarded_traffic=allow_forwarded_traffic,
        allow_virtual_network_access=allow_virtual_network_access,
        allow_gateway_transit=allow_gateway_transit,
        **peering_kwargs
    )

    # This is a special case where one side of the peering has been deleted and recreated.
    # In order to establish the new peer, the remote peer needs to be set back to "Initiated" state.
    if peering.get('error', '').startswith('Azure Error: RemotePeeringIsDisconnected'):
        rname_match = re.search('because remote peering (\S+) referencing parent virtual network', peering['error'])
        remote_name = rname_match.group(1).split('/')[-1]

        remote_peering = await hub.exec.azurerm.network.virtual_network_peering.get(
            name=remote_name,
            virtual_network=remote_virtual_network,
            resource_group=remote_vnet_group,
            azurearm_log_level='info',
            **connection_auth
        )

        remote_peering_kwargs = remote_peering.copy()
        remote_peering_kwargs.update(connection_auth)
        remote_peering_kwargs['peering_state'] = 'Initiated'
        remote_peering_kwargs.pop('remote_virtual_network')

        remote_peering = await hub.exec.azurerm.network.virtual_network_peering.create_or_update(
            remote_virtual_network=virtual_network,
            remote_vnet_group=resource_group,
            virtual_network=remote_virtual_network,
            resource_group=remote_vnet_group,
            **remote_peering_kwargs
        )

        peering = await hub.exec.azurerm.network.virtual_network_peering.create_or_update(
            name=name,
            remote_virtual_network=remote_virtual_network,
            remote_vnet_group=remote_vnet_group,
            virtual_network=virtual_network,
            resource_group=resource_group,
            use_remote_gateways=use_remote_gateways,
            allow_forwarded_traffic=allow_forwarded_traffic,
            allow_virtual_network_access=allow_virtual_network_access,
            allow_gateway_transit=allow_gateway_transit,
            **peering_kwargs
        )

    if 'error' not in peering:
        ret['result'] = True
        ret['comment'] = 'Peering object {0} has been created.'.format(name)
        return ret

    ret['comment'] = 'Failed to create peering object {0}! ({1})'.format(name, peering.get('error'))
    return ret


async def absent(hub, ctx, name, virtual_network, resource_group, connection_auth=None):
    '''
    .. versionadded:: 1.0.0

    Ensure a virtual network peering object does not exist in the virtual network.

    :param name:
        Name of the peering object.

    :param virtual_network:
        Name of the existing virtual network containing the peering object.

    :param resource_group:
        The resource group assigned to the virtual network.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    peering = await hub.exec.azurerm.network.virtual_network_peering.get(
        name,
        virtual_network,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' in peering:
        ret['result'] = True
        ret['comment'] = 'Peering object {0} was not found.'.format(name)
        return ret

    elif ctx['test']:
        ret['comment'] = 'Peering object {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': peering,
            'new': {},
        }
        return ret

    deleted = await hub.exec.azurerm.network.virtual_network_peering.delete(
        name,
        virtual_network,
        resource_group,
        **connection_auth
    )

    if deleted:
        ret['result'] = True
        ret['comment'] = 'Peering object {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': peering,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete peering object {0}!'.format(name)
    return ret
